-- https://www.rcgroups.com/forums/showthread.php?3181709-Heli-Model-for-Taranis
-- https://ardupilot.org/copter/docs/traditional-helicopter-swashplate-setup.html

-- angles defined with direction of forward flight being zero degrees
-- anticlockwise is positive angle

-- In OpenTX, CYC1 is the servo in line with the elevator (forward/backwards line of the heli).
-- CYC2 is CCW around from the CYC1 and CYC3 is CCW around from CYC2.
-- In ArduPilot, documentation is given on the heads types used.
-- They use SERVO1, SERVO2, SERVO 3 and SERVO5.
-- I prefer to use ARDUPILOT SERVO LAYOUT for head SERVOS.
-- Just abbreviated from SERVO to SRV to fit in the 4 char name labels.

-- The OpenTX X120 headtype is not supported anymore.
-- A gliding scale of possible angles between 0 and 179 is used to shift between the head types.
-- This covers most commonly used headtypes.
-- X120 doesn't fit in that system.

-- With the new system to enter head types, the common 135° is now also supported, as well as the 140°.
-- I have a strong suspicion the OpenTX 140 head config is NOT 140°, but 135°.
-- This because it is only for 135° that the scaling for the lateral is exactly as it is in OpenTX.
-- (45° from the top to both sides)

-- Mapping from OpenTX to Edge HELI.lua for 120°
-- CYC1 = SRV3
-- CYC2 = SRV1
-- CYC3 = SRV2


local P = {}                    --helper arithmetic functions "library"

function P.cos(DEG)
  return math.cos(math.rad(DEG))
end

function P.sin(DEG)
  return math.sin(math.rad(DEG))
end

function P.tan(DEG)
  return math.tan(math.rad(DEG))
end


local script = {}                                         -- script function "library"

script.input =
    {
        { "SWASH_DEG",  VALUE,  0, 179, 120 },            -- see script
        { "RING",       VALUE,  0, 100, 100},             -- "RING" radius % "length" of LAT_src/LON_src
        { "EXPO",       VALUE,  -100, 100, 0},            -- expo on "length" of LAT_src and LON_src
        { "LON_src",    SOURCE, getFieldInfo("ele").id}, -- LON source, normally Elevator
        { "LAT_src",    SOURCE, getFieldInfo("ail").id}, -- LAT source, normally Aileron
        { "PITCH_src",  SOURCE, getFieldInfo("thr").id}, -- PITCH source, normally Throttle
    }

script.output = 
    {   
        "SRV1",
        "SRV2",
        "SRV3",
        "SRV5",   --attention, no SRV4 !!!
    }

function script.run(SWASH_DEG,RATE,EXPO,LON_src, LAT_src, PITCH_src)
  --scaling to fraction of 1 (100%)
  RATE = RATE/100
  EXPO = EXPO/100
  local LON = LON_src/1024
  local LAT = LAT_src/1024
  local PITCH = PITCH_src/1024
  
  --convert to radial coordinates for rate and expo
  local Length = math.sqrt(LON*LON + LAT*LAT)
  local Angle = math.atan2(LON,LAT)
  
  --apply max rate
  if RATE > 0 then
    Length = math.min(Length,RATE)
  end
  
  --apply expo: y = e * x^3 + (1-e) * x
  Length = EXPO * math.pow(Length,3) + (1-EXPO) * Length
  
  --return to cartesian coordinates LON, LAN
  LON = Length * math.sin(Angle)
  LAT = Length * math.cos(Angle)
  
  --the actual calculations on the cyclic mixing.
  local SRV1, SRV2, SRV3, SRV5
  if SWASH_DEG == 0 then
    --no CCPM head used, just map the inputs on the outputs. 
    --useful for x,y 'length' limiter and expo for all purposes
    SRV1 = LON
    SRV2 = LAT
    SRV3 = PITCH
    SRV5 = 0
  
  elseif SWASH_DEG < 90 then 
    -- 4 servo X-head, SRV1 DEG 1°-89°
    -- SRV1 DEG, SRV2 -DEG
    -- SRV5 180° + DEG, SRV3 180° - DEG
    -- Longitudinal is mapped to 100% LON_src
    -- Lateral is scaled down to give same head tilt as longitudinal on 100% LAT_src
    SRV1 = PITCH + LON - LAT*P.tan(SWASH_DEG)
    SRV2 = PITCH + LON + LAT*P.tan(SWASH_DEG)
    SRV5 = PITCH - LON + LAT*P.tan(SWASH_DEG)
    SRV3 = PITCH - LON - LAT*P.tan(SWASH_DEG)
  
  elseif SWASH_DEG == 90 then 
    -- 4 servo head, SRV1 DEG 90°
    -- SRV1 90°, SRV2 -90°
    -- SRV5 0°, SRV3 180°
    SRV1 = PITCH + LAT
    SRV2 = PITCH - LAT
    SRV5 = PITCH + LON
    SRV3 = PITCH - LON
  
  elseif SWASH_DEG < 135 then 
    -- 3 servo head, SRV1 180°-DEG = (46°-89°)
    -- SRV1 180° - DEG, SRV2 180° + DEG
    -- SRV3 180°
    -- Longitudinal SRV1 / SRV2 is scaled down to give same head tilt as longitudinal on SRV3
    -- Lateral SRV1 / SRV2 is scaled down to give same head tilt as longitudinal on 100% LAT_src
    SRV1 = PITCH + LON * P.cos(180 - SWASH_DEG) + LAT * P.sin(180 - SWASH_DEG) 
    SRV2 = PITCH + LON * P.cos(180 - SWASH_DEG) - LAT * P.sin(180 - SWASH_DEG) 
    SRV3 = PITCH - LON
    SRV5 = 0
        
  elseif SWASH_DEG < 180 then 
    -- 3 servo head, SRV1 180°-DEG = (1°-45°)
    -- SRV1 180° - DEG, SRV2 180° + DEG
    -- SRV3 180°
    -- Longitudinal SRV1 / SRV2 is mapped on 100% as the longitudal is matched to SRV3
    -- Lateral SRV1 / SRV2 is scaled up to give same head tilt as longitudinal on 100% LAT_src
    SRV1 = PITCH + LON + LAT / P.tan(180 - SWASH_DEG) 
    SRV2 = PITCH + LON - LAT / P.tan(180 - SWASH_DEG) 
    SRV3 = PITCH - LON
    SRV5 = 0
  
  end
  
  return SRV1*1024, SRV2*1024, SRV3*1024, SRV5*1024
end

return script